%% Project:   High Angular Resolution Diffusion Imaging Tools
% Point Spread Function (PSF), computed as a Gaussian function.
% The profile shape of this PSF resembles the truncated theoretical solution                                                                         
%                                                                                              
%   Language:  MATLAB(R)
%   Author:  Erick Canales-Rodríguez, Lester Melie-García, Yasser Iturria-Medina, Yasser Alemán-Gómez
%   Date: 2013, Version: 1.2          
% 
% --- Some comments ----
% Notice that the theoretical/experimental PSF is just the fourier transform of the q-space points 
% where the signal was measured (cartesian sampling scheme). That is, the FFT of a 3D binary image 
% with values equal to 1 for those pixels where the signal was measured in q-space, and zero elsewhere.
% However, the deconvolution using the resulting PSF is very unstable and noisy due to the many sidelobes 
% of this PSF (including negative sidelobes) and the low resolution of the data.
% In addition, due to the discretization, the obtained PSF is not smooth.
%
% In order to improve the deconvolution procedure in DSI, we can use instead the gaussian function that best fit 
% the theoretical PSF, which is smooth and definite positive. It does not contain unwanted oscillations.
% We have found that the deconvolution using this PSF produces high quality results for the standard DSI sampling scheme
% (515 points and bmax=7000-9000 s/mm^2).
% Note that if a different sampling scheme is used, the optimal gaussian PSF must be recomputed/obtained.
%
% A different practical consideration that I have used is to preserve only the main positive lobe of the PSF, 
% due to this part of the function is the main responsible of the unwanted convolution effect in DSI.
% This alternative PSF is computed by the function: create_mainlobe_PSF  
%
% See also create_mainlobe_PSF, test_DSI_example, SignalMatrixBuilding.


function [PSF, Sampling_grid] = create_gaussian_PSF(qc,Resolution)
X = -(Resolution-1)/2:(Resolution-1)/2;
X = X*10^-3;% (in mm)
Y = X; Z = X;
[Xnew Ynew Znew] = meshgrid(X,Y,Z);
Diso = 0.1*10^-3;% (in mm^2/s)
t = 20*10^-3;% (in s)
R2 = Xnew.^2 + Ynew.^2 + Znew.^2;
PSF = ((1/sqrt(4*pi*Diso*t))^3)*exp(-(1/(4*Diso*t))*R2);
% ---
Sampling_grid = zeros(Resolution,Resolution,Resolution);
n = length(qc);
for i=1:n
    Sampling_grid(qc(i,1),qc(i,2),qc(i,3)) = 1;
end

% Sampling_grid = zeros(Resolution,Resolution,Resolution);
% Center = (Resolution + 1)/2;
% qmax = 5;
% Range  = Center-qmax:Center+qmax;
% Sampling_grid(Range,Range,Range) = 1;
% ---

% --- Thresholding
PSF = PSF.*Sampling_grid; % Truncated PSF
PSF = PSF/sum(PSF(:));
end
