%% Project:   High Angular Resolution Diffusion Imaging Tools
% Point Spread Function (PSF), defined by the main lobe of the experimental PSF (truncated theoretical solution).
%                                                                                              
%   Language:  MATLAB(R)
%   Author:  Erick Canales-Rodríguez, Lester Melie-García, Yasser Iturria-Medina, Yasser Alemán-Gómez
%   Date: 2013, Version: 1.2          
% 
% --- Some comments ----
% Notice that the theoretical/experimental PSF is just the fourier transform of the q-space points 
% where the signal was measured (cartesian sampling scheme). That is, the FFT of a 3D binary image 
% with values equal to 1 for those pixels where the signal was measured in q-space, and zero elsewhere.
% However, the deconvolution using the resulting PSF is very unstable and noisy due to the many sidelobes 
% of this PSF (including negative sidelobes) and the low resolution of the data.
% In addition, due to the discretization, the obtained PSF is not smooth.
%
% In order to improve the deconvolution procedure in DSI, a practical consideration that I have used 
% is to preserve only the main lobe of the PSF, which is positive, due to this part of the function is the main 
% responsible of the unwanted convolution effect in DSI.
% Note that the truncation level (radius) depends on the experimental sampling scheme.
% The truncation implemented here in this code is for a standard DSI sampling scheme
% with 515 points and bmax=7000-9000 s/mm^2
% If you are using a different sampling scheme, then a different truncation radius must be used for optimal results. 
%
% A different alternative is to use instead the gaussian function that best fit the theoretical PSF.
% Under this approximation the PSF is smooth and definite positive by definition. It does not contain unwanted oscillations.
% We have found that the deconvolution using this PSF also produces high quality results.
% This alternative PSF is computed by the function: create_gaussian_PSF  
%
% See also create_gaussian_PSF, test_DSI_example, SignalMatrixBuilding.


function [PSF, Sampling_grid] = create_mainlobe_PSF(qc,Resolution)
Sampling_grid = zeros(Resolution,Resolution,Resolution);
n = length(qc);
for i=1:n
    Sampling_grid(qc(i,1),qc(i,2),qc(i,3)) = 1;
end

% --- Experimental PSF
PSF  = (real(fftshift(fftn(ifftshift(Sampling_grid)))));
PSF(PSF<0)=0;

% --- Thresholding
PSF = PSF.*Sampling_grid; % Truncated PSF
PSF = PSF/sum(PSF(:));
end
