%% Project:   High Angular Resolution Diffusion Imaging Tools
% deconv_RL_TV_3D Deblur a 3D image using Lucy-Richardson method with
%                 With Total Variation (TV) Regularization.
%
% J = deconv_RL_TV_3D(I, PSF, Niter, W, type_of_noise) deconvolves image I 
%   using Lucy-Richardson algorithm, returning deblurred image J. 
%   The assumption is that the image I was created by convolving a true image
%   with a point-spread function PSF and possibly by adding noise.
%   
%   Niter (optional) is the number of iterations (default is 10)
%
%   W is a WEIGHT (optional) value assigned to each pixel to reflect its 
%   recording quality. A bad pixel is excluded from the solution by
%   assigning it zero weight value. Instead of giving a weight of one for
%   good pixels, you can adjust their weight according to the amount of
%   flat-field correction. Default is a unit array of the same size as 
%   input image I.
%
%   type_of_noise is the distribution of the noise.
%   The current implementation supports two different options:
%   'Poisson' (default) or 'Gaussian'.
%
%   References
%   ----------
%   "Acceleration of iterative image restoration algorithms, by D.S.C. Biggs 
%   and M. Andrews, Applied Optics, Vol. 36, No. 8, 1997.
%
%   "Deconvolutions of Hubble Space Telescope Images and Spectra",
%   R.J. Hanisch, R.L. White, and R.L. Gilliland. in "Deconvolution of Images 
%   and Spectra", Ed. P.A. Jansson, 2nd ed., Academic Press, CA, 1997.
%   
%   "Richardson–Lucy Algorithm With Total Variation Regularization for 3D Confocal 
%   Microscope Deconvolution". NICOLAS DEY, LAURE BLANC-FERAUD, CHRISTOPHE ZIMMER, 
%   PASCAL ROUX, ZVI KAM, JEAN-CHRISTOPHE OLIVO-MARIN, AND JOSIANE ZERUBIA.
%   MICROSCOPY RESEARCH AND TECHNIQUE 69:260–266 (2006).
%   
%   "Application of regularized Richardson–Lucy algorithm for deconvolution of 
%    confocal microscopy images". M. LAASMAA, M. VENDELIN & P. PETERSON.
%    Journal ofMicroscopy, Vol. 243, Pt 2 2011, pp. 124–140.
%                                                                                                
% Language:  MATLAB(R)
% Author:  Erick Canales-Rodríguez, Lester Melie-García, Yasser Iturria-Medina, Yasser Alemán-Gómez
% Date: 2013, Version: 1.2          
% 
% See also, deconv_RL_3D, deconv_RLdamp_3D, deconvlucy

function [Pd]= deconv_RL_TV_3D(P, PSF, Niter, W, type_of_noise)

sizeP = size(P);
H_PSF = (real(fftshift(ifftn(ifftshift(PSF)))));
Pd = P;
Pzero = zeros(sizeP);

switch nargin
    case 2
        Niter = 10;
        W = ones(sizeP);
        type_of_noise = 'Poisson';
    case 3
        W = ones(sizeP);
        type_of_noise = 'Poisson';
    case 4
        type_of_noise = 'Poisson';
end

if strcmp(type_of_noise, 'Gaussian');
    KK = reblurred(H_PSF,P);
    lambda = 1/20;
    a = 2; % acceleration factor
end

if strcmp(type_of_noise, 'Poisson');
    lambda = 1/15;
    a = 1; % no acceleration
end

epsilon = 1e-3;

for i = 1:Niter
    Pi = Pd;
    % -- R-L --
    if strcmp(type_of_noise, 'Poisson');
        K = reblurred(H_PSF,Pi);
        RL_factor = reblurred(H_PSF,P./(K + eps) + eps);
    end
    if strcmp(type_of_noise, 'Gaussian');
        K = reblurred(H_PSF,reblurred(H_PSF,Pi)) + eps;
        RL_factor = KK./K;
    end
    % -- TV ---
    Gr = grad(Pi);
    d = sqrt(sum(Gr.^2,4));
    div_f = div( Gr ./ repmat( sqrt( epsilon^2 + d.^2 ) , [1 1 1 3]) );
    G0 = abs( 1 - lambda*div_f ); % abs -> to avoid negative values
    TV_factor =  1./(G0 + eps); % eps -> to avoid zero values
    % ---------
    Pi_1 = Pi.*(RL_factor.*TV_factor).^a;
    Pi_1 = max(Pzero, Pi_1); % positivity
    Pi_1 = Pi_1.*W;
    Pi_1 = Pi_1*sum(P(:))/sum(Pi_1(:)); % energy preservation
    % ---------
    Pd = Pi_1; 
end
end

% --- private functions -----

function K = reblurred(H_PSF,Pi)
    H_Pi  = (real(fftshift(ifftn(ifftshift(Pi)))));
    K = (real(fftshift(fftn(ifftshift( H_PSF.*H_Pi )))));
end

function [fxyz] = grad(M)
% grad - gradient operator
fx = M([2:end end],:,:)-M;
fy = M(:,[2:end end],:)-M;
fz = M(:,:,[2:end end])-M;

fxyz = cat(4,fx,fy,fz);
end

function fd = div(P)
% div - divergence operator

Px = squeeze(P(:,:,:,1));
Py = squeeze(P(:,:,:,2));
Pz = squeeze(P(:,:,:,3));

fx = Px-Px([1 1:end-1],:,:);
fx(1,:,:)   = Px(1,:,:);    % boundary
fx(end,:,:) = -Px(end-1,:,:);
% ---
fy = Py-Py(:,[1 1:end-1],:);
fy(:,1,:)   = Py(:,1,:);    % boundary
fy(:,end,:) = -Py(:,end-1,:);
% ---
fz = Pz-Pz(:,:,[1 1:end-1]);
fz(:,:,1)   = Pz(:,:,1);    % boundary
fz(:,:,end) = -Pz(:,:,end-1);
% ---
fd = fx+fy+fz;
end
