%% Project:   High Angular Resolution Diffusion Imaging Tools
% deconv_RLdamp_3D Deblur a 3D image using Damped Lucy-Richardson method.
%
% J = deconv_RLdamp_3D(I, PSF, Niter, W, type_of_noise) deconvolves image I 
%   using Lucy-Richardson algorithm, returning deblurred image J. 
%   The assumption is that the image I was created by convolving a true image
%   with a point-spread function PSF and possibly by adding noise.
%   
%   Niter (optional) is the number of iterations (default is 10)
%
%   W is a WEIGHT (optional) value assigned to each pixel to reflect its 
%   recording quality. A bad pixel is excluded from the solution by
%   assigning it zero weight value. Instead of giving a weight of one for
%   good pixels, you can adjust their weight according to the amount of
%   flat-field correction. Default is a unit array of the same size as 
%   input image I.
%
%   type_of_noise is the distribution of the noise.
%   The current implementation supports two different options:
%   'Poisson' (default) or 'Gaussian'.
%
%   References
%   ----------
%   "Acceleration of iterative image restoration algorithms, by D.S.C. Biggs 
%   and M. Andrews, Applied Optics, Vol. 36, No. 8, 1997.
%
%   "Deconvolutions of Hubble Space Telescope Images and Spectra",
%   R.J. Hanisch, R.L. White, and R.L. Gilliland. in "Deconvolution of Images 
%   and Spectra", Ed. P.A. Jansson, 2nd ed., Academic Press, CA, 1997.
%
%   "Image Restoration Using the Damped Richardson-Lucy Method", by Richard
%   L. White. The Restoration of HST Images and Spectra II Space Telescope 
%   Science Institute, 1994 R. J. Hanisch and R. L. White, eds.
%
%   "A Modified Damped Richardson Lucy Algorithm to Reduce Isotropic Background
%    Effects in Spherical Deconvolution". Flavio Dell’Acqua, Paola Scifo, 
%    Giovanna Rizzo, Marco Catani, Andrew Simmons, Giuseppe Scotti, Ferruccio Fazio.
%    NeuroImage, 2009.
%    
%                                                                                                
% Language:  MATLAB(R)
% Author:  Erick Canales-Rodríguez, Lester Melie-García, Yasser Iturria-Medina, Yasser Alemán-Gómez
% Date: 2013, Version: 1.2          
% 
% See also, deconv_RL_3D, deconv_RL_TV_3D, deconvlucy

function Pd = deconv_RLdamp_3D(P, PSF, Niter, W, type_of_noise)
sizeP = size(P);
H_PSF = (real(fftshift(ifftn(ifftshift(PSF)))));
Pd = P;
Pzero = zeros(sizeP);

switch nargin
    case 2
        Niter = 10;
        W = ones(sizeP);
        type_of_noise = 'Poisson';
    case 3
        W = ones(sizeP);
        type_of_noise = 'Poisson';
    case 4
        type_of_noise = 'Poisson';
end

if strcmp(type_of_noise, 'Poisson');
    T  = 0.005;
    N = 1.1;
    a = 1.1; % acceleration factor
    for i = 1:Niter
        Pi = Pd;
        K = reblurred(H_PSF,Pi);
        % ---
        Rk = ( -2/T )*(P.*log(K./(P + eps)) - K + P);
        R = min(ones(size(Rk)),Rk);
        U = (R.^(N-1)).*(N - (N-1)*R);
        Func = 1 + U.*( (P - K)./(K + eps) );
        % ---
        RL_factor = reblurred(H_PSF,Func).^a;
        Pi_1 = max(Pzero, Pi.*RL_factor); % positivity
        Pi_1 = Pi_1.*W;
        Pi_1 = Pi_1*sum(P(:))/sum(Pi_1(:)); % energy preservation
        Pd = Pi_1;
    end
end

if strcmp(type_of_noise, 'Gaussian');
    KK = reblurred(H_PSF,P);
    mu = 0.5;
    expo = 8;
    nu = 0.001;
    % ---
    a = 2; % acceleration factor
    for i = 1:Niter
        Pi = Pd;
        K = reblurred(H_PSF,reblurred(H_PSF,Pi)) + eps;
        R = 1 - ( Pi.^expo )./( Pi.^expo + (max(Pi(:))*nu)^expo );
        U = 1 - mu*R;
        RL_factor = ( 1 + U.*( (KK-K)./K ) ).^a; 
        Pi_1 = max(Pzero, Pi.*RL_factor); % positivity
        Pi_1 = Pi_1.*W;
        Pi_1 = Pi_1*sum(P(:))/sum(Pi_1(:)); % energy preservation
        Pd = Pi_1;
    end
end

end

% --- private functions -----

function K = reblurred(H_PSF,Pi)
    H_Pi  = (real(fftshift(ifftn(ifftshift(Pi)))));
    K = (real(fftshift(fftn(ifftshift( H_PSF.*H_Pi )))));
end
