%% Project:   High Angular Resolution Diffusion Imaging Tools
% deconv_RL_1D Lucy-Richardson method for spherical deconvolution.
%
% J = deconv_RL_1D(I, Kernel, fODF0, Niter, type_of_noise) deconvolves vector I 
%   using Lucy-Richardson algorithm, returning deblurred vector J. 
%   The assumption is that the vector I was created by convolving a true
%   vector with a Kernel and possibly by adding noise.
%
%   fODF0 is an initial estimator for fODF. 
%         Two possible options are: (1) Constant value on the sphere:
%                                       "non-negative iso-probable spherical function"
%                                   (2) Tikhonov solution ? (to be explored)
%   
%   Niter (optional) is the number of iterations (default is 10)
%
%   type_of_noise is the distribution of the noise.
%   The current implementation supports two different options:
%   'Poisson' (default) or 'Gaussian'.
%
%   References
%   ----------
%   "Acceleration of iterative image restoration algorithms, by D.S.C. Biggs 
%   and M. Andrews, Applied Optics, Vol. 36, No. 8, 1997.
%
%   "Deconvolutions of Hubble Space Telescope Images and Spectra",
%   R.J. Hanisch, R.L. White, and R.L. Gilliland. in "Deconvolution of Images 
%   and Spectra", Ed. P.A. Jansson, 2nd ed., Academic Press, CA, 1997.
%
%   "Image Restoration Using the Damped Richardson-Lucy Method", by Richard
%   L. White. The Restoration of HST Images and Spectra II Space Telescope 
%   Science Institute, 1994 R. J. Hanisch and R. L. White, eds.
%
%   "A Modified Damped Richardson Lucy Algorithm to Reduce Isotropic Background
%    Effects in Spherical Deconvolution". Flavio Dell’Acqua, Paola Scifo, 
%    Giovanna Rizzo, Marco Catani, Andrew Simmons, Giuseppe Scotti, Ferruccio Fazio.
%    NeuroImage, 2009.
%
%                                                                                                
% Language:  MATLAB(R)
% Author:  Erick Canales-Rodríguez, Lester Melie-García, Yasser Iturria-Medina, Yasser Alemán-Gómez
% Date: 2013, Version: 1.2          
% 
% See also, deconv_RLdamp_1D

function [fODF, fgm, fcsf] = deconv_RL_1D(Signal, Kernel, fODF0, Niter, type_of_noise)
% fODF: purely anisotropic part
% fgm : fraction of the isotropic component for GM
% fcsf: fraction of the isotropic component for CSF

switch nargin
    case 2
        % -- Initial fODF0: Constant value on the sphere 
        %    "non-negative iso-probable spherical function"
        fODF0 = ones(size(Kernel,2));
        fODF0 = fODF0/sum(fODF0);
        % ---------
        Niter = 200;
        type_of_noise = 'Gaussian';
    case 3
        Niter = 200;
        type_of_noise = 'Gaussian';
    case 4
        type_of_noise = 'Gaussian';
end

fODF = fODF0;
fzero = 0;
% fzero = 1e-06;
KernelT = Kernel'; % only one time

if strcmp(type_of_noise, 'Poisson');
    for i = 1:Niter
        fODFi = fODF;
        Reblurred = Kernel*fODFi + eps;
        RL_factor = KernelT*(Signal./Reblurred);
        fODF = max(fzero, fODFi.*RL_factor); % positivity
    end
end


if strcmp(type_of_noise, 'Gaussian');
    KernelS = KernelT*Signal;
    for i = 1:Niter
        fODFi = fODF;
        RL_factor = KernelS./( KernelT*(Kernel*fODFi) + eps);
        fODF = max(fzero, fODFi.*RL_factor); % positivity
    end
end

% ------------------------------------------------------------------
fODF = fODF/sum(fODF); % energy preservation

nc = length(fODF);
fgm  = fODF(nc);       % fraction of the isotropic component for GM
fcsf = fODF(nc-1);     % fraction of the isotropic component for CSF
fODF(nc-1:nc) = [];    % purely anisotropic part
end
