%% Project:   High Angular Resolution Diffusion Imaging Tools
% deconv_RLdamp_1D damped Lucy-Richardson method for spherical deconvolution.
%
% J = deconv_RLdamp_1D(I, Kernel, fODF0, Niter, type_of_noise) deconvolves vector I 
%   using damped Lucy-Richardson algorithm, returning deblurred vector J. 
%   The assumption is that the vector I was created by convolving a true
%   vector with a Kernel and possibly by adding noise.
%
%   fODF0 is an initial estimator for fODF. 
%         Two possible options are: (1) Constant value on the sphere:
%                                       "non-negative iso-probable spherical function"
%                                   (2) Tikhonov solution ? (to be explored)
%   
%   Niter (optional) is the number of iterations (default is 10)
%
%   type_of_noise is the distribution of the noise.
%   The current implementation supports two different options:
%   'Poisson' (default) or 'Gaussian'.
%
%   References
%   ----------
%   "Acceleration of iterative image restoration algorithms, by D.S.C. Biggs 
%   and M. Andrews, Applied Optics, Vol. 36, No. 8, 1997.
%
%   "Deconvolutions of Hubble Space Telescope Images and Spectra",
%   R.J. Hanisch, R.L. White, and R.L. Gilliland. in "Deconvolution of Images 
%   and Spectra", Ed. P.A. Jansson, 2nd ed., Academic Press, CA, 1997.
%
%   "Image Restoration Using the Damped Richardson-Lucy Method", by Richard
%   L. White. The Restoration of HST Images and Spectra II Space Telescope 
%   Science Institute, 1994 R. J. Hanisch and R. L. White, eds.
%
%   "A Modified Damped Richardson Lucy Algorithm to Reduce Isotropic Background
%    Effects in Spherical Deconvolution". Flavio Dell’Acqua, Paola Scifo, 
%    Giovanna Rizzo, Marco Catani, Andrew Simmons, Giuseppe Scotti, Ferruccio Fazio.
%    NeuroImage, 2009.
%
%                                                                                                
% Language:  MATLAB(R)
% Author:  Erick Canales-Rodríguez, Lester Melie-García, Yasser Iturria-Medina, Yasser Alemán-Gómez
% Date: 2013, Version: 1.2          
%
% See also, deconv_RL_1D

function [fODF, fgm, fcsf] = deconv_RLdamp_1D(Signal, Kernel, fODF0, Niter, type_of_noise)
% fODF: purely anisotropic part
% fgm : fraction of the isotropic component for GM
% fcsf: fraction of the isotropic component for CSF

switch nargin
    case 2
        % -- Initial fODF0: Constant value on the sphere 
        %    "non-negative iso-probable spherical function"
        fODF0 = ones(size(Kernel,2));
        fODF0 = fODF0/sum(fODF0);
        % ---------
        Niter = 200;
        type_of_noise = 'Gaussian';
    case 3
        Niter = 200;
        type_of_noise = 'Gaussian';
    case 4
        type_of_noise = 'Gaussian';
end

fODF = fODF0;
KernelT = Kernel'; % only one time

if strcmp(type_of_noise, 'Poisson');
    fzero = 1e-06;
    a = 0.8;
    T = 0.15; N = 1/2;
    for i = 1:Niter
        Reblurred = Kernel*fODF;
        % --- approach: L. White
        Rkk = ( -2/T )*(Signal.*log(Reblurred./(Signal + eps)) - Reblurred + Signal);
        R = min(1, Rkk);
        Uk = (R.^(N-1)).*(N - (N-1)*R);
        % ------        
        K = 1 + Uk.*( (Signal - Reblurred)./(Reblurred + eps) ); 
        RL_factor = (KernelT*K).^a;
        fODF = max(fzero, fODF.*RL_factor); % positivity
    end
end

if strcmp(type_of_noise, 'Gaussian');
    fzero = 1e-06;
    KernelS = KernelT*Signal;
    % -----
    % a = 1.2; % acceleration factor (if a==1 no acceleration)
    V = 8;
    % nu = 0.05; % optimal value for b=1500 in the range [0.04 0.06]
    nu = 0.08; % optimal value for b=3000 in the range [0.06 0.1];
    mu = max(0, 1 - 4*std(Signal) );
    nuV = nu^V;
    for i = 1:Niter
        % --- approach: Flavio Dell’Acqua
        Dem = KernelT*(Kernel*fODF);
        fODFV = fODF.^V;
        Rk = 1 - (fODFV)./(fODFV + nuV);
        Uk = 1 - mu*Rk;
        RL_factor = 1 + Uk.*( (KernelS-Dem)./(Dem + eps) );
        % RL_factor = (RL_factor).^a; % acceleration for fast convergence
        fODF = max(fzero, fODF.*RL_factor); % positivity
    end
end

% ------------------------------------------------------------------
fODF = fODF/sum(fODF); % energy preservation

nc = length(fODF);
fgm  = fODF(nc);       % fraction of the isotropic component for GM
fcsf = fODF(nc-1);     % fraction of the isotropic component for CSF
fODF(nc-1:nc) = [];    % purely anisotropic part
end
